#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a drawing look usinmg G'mic Stencil

# (c) Dinasset 2021
#
#   History:
#
#   v1.0: 2021-01-30: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

 
from gimpfu import *
import random
from random import randint
import sys, os
import time
import datetime

# define here some constant value, when required
FixWH=3000

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

####################
#   M A I N
#-------------------
    
def DrawStyleG (inImage, inDrawable, outimgWH, Mask, linesdir,
                     doFlatten) :
    
    pdb.gimp_context_push()
    currentFG = pdb.gimp_context_get_foreground()
    # source image characteristics
    width = inImage.width
    height = inImage.height
    OrigWH = width+height
    otype = inImage.base_type
    # image name
    iname = inImage.name
    split = iname.split("(", 1)
    iname = split[0]
    end = len(iname)-2
    iname = iname[1:end]    # remove []
    #info_msg (iname)
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    
    Ratio = FixWH*1.00/OrigWH*1.00
    rWidth = int(width*Ratio)
    rHeight = int(height*Ratio)
    WorkWH = rWidth+rHeight
    
    # START    
    # create output image
    newImage = pdb.gimp_image_new (rWidth, rHeight, otype )
    newBG = pdb.gimp_layer_new_from_drawable (inDrawable, newImage)
    newImage.add_layer (newBG, 0)
    pdb.gimp_layer_scale (newBG, rWidth, rHeight, False)
    newBG.name = inDrawable.name
    if '.' in newBG.name:
        split = newBG.name.split(".", 1)
        newBG.name = split[0]
    # ofnuts technique to give it a name (unsaved). I set it to the name of the original layer
    newImage.filename=os.path.join(gimp.user_directory(4),iname)
    
    #pdb.gimp_display_new (newImage)
    #return()
     
    # check lines direction
    vertical=False
    if linesdir==2: # vertical
        pdb.gimp_image_rotate (newImage, ROTATE_90)
        vertical=True
     
    # Change colour to a light pastel nuance
    normalized_layer=newBG.copy()
    newImage.add_layer (normalized_layer, 0)
    normalized_layer.name="Normalized"
    pdb.gimp_drawable_curves_spline (normalized_layer, HISTOGRAM_VALUE, 6,
                                     ( 0., 0.25, 0.5, 0.625, 1.0, 1.0)) 
    pdb.gimp_selection_none(newImage)
    pdb.gimp_drawable_hue_saturation(normalized_layer, HUE_RANGE_ALL, 0.0, 0.0, 100.0, 0.0)
    
    #pdb.gimp_display_new (newImage)
    #return()
    
    # Apply G'MIC Stencil
    pdb.plug_in_gmic_qt(newImage, normalized_layer, 1,2 ,
    #[G'MIC] Stencil: fx_stencil 1.5,0,8,0,2,1,0,50,50
    "-v - -fx_stencil 1.5,0,8,0,2,1,0,50,50 ")
    stencil_layer = newImage.layers[0]
    stencil_layer.name = "Stencil"
    stencil_layer.mode = LAYER_MODE_LINEAR_BURN
    stencil_layer.opacity = 95.0
    #pdb.plug_in_spread (newImage, stencil_layer, 1.0, 1.0)
    pdb.gimp_drawable_brightness_contrast (stencil_layer, -0.4, +0.4)
    
    #pdb.gimp_display_new (newImage)
    #return()

    if linesdir>0: # if not *none (otherwise let them to be curved)    
    # Apply Linear Blur
        LINEAR=0
        pdb.plug_in_mblur (newImage, stencil_layer, LINEAR,
                       10.0, 0.0, 0.0, 0.0)    
    
    #pdb.gimp_display_new (newImage)
    #return()
     
    # Apply G'MIC Dream Smoothing
    pdb.plug_in_gmic_qt(newImage, stencil_layer, 1,2 ,
    #[G'MIC] Dream Smoothing: fx_dreamsmooth 3,1,1,0.8,0,0.8,1,24,0
    "-v - -fx_dreamsmooth 2,1,1,0.8,0,0.8,1,24,0 ") # 14.1 chg from 1 to 2
    dream_layer = newImage.layers[0]
    dream_layer.name = "Dream"
    dream_layer.mode = LAYER_MODE_SUBTRACT
    dream_layer.opacity = 55.0    #95.0  chg 14.1 from 95 to 55
    pdb.plug_in_spread (newImage, dream_layer, 1.0, 1.0)
    
    #pdb.gimp_display_new (newImage)
    #return()
    
    # create the border with Neon
    borders_layer=normalized_layer.copy()
    newImage.add_layer (borders_layer, 0)
    borders_layer.name="Neon Borders"
    borders_layer.mode=LAYER_MODE_GRAIN_MERGE
    RADIUS=2.5
    AMOUNT=1.0
    pdb.gimp_layer_flatten (borders_layer)   
    pdb.plug_in_neon (newImage, borders_layer,
                      RADIUS, AMOUNT)
    pdb.gimp_drawable_curves_spline (borders_layer, 0, 6,
                            (0.0,0.4, 0.8,0.6, 1.0,1.0))  
    pdb.gimp_layer_add_alpha (borders_layer)
    pdb.gimp_drawable_desaturate (borders_layer, DESATURATE_LUMA)
    
    # duplicate the borders  
    borders2_layer=borders_layer.copy()
    newImage.add_layer (borders2_layer, 0)
    borders2_layer.name="2.nd Neon Borders"
    borders2_layer.opacity = 100.0
    borders2_layer.mode=LAYER_MODE_PIN_LIGHT
    RLE=1
    pdb.plug_in_gauss (newImage, borders2_layer, rWidth/100, rHeight/100, RLE)
    pdb.plug_in_vinvert (newImage, borders2_layer)
    
    #pdb.gimp_display_new (newImage)
    #return()
     
# =======================================================================================
        
    strokeMode = " stroked using stencil, neon"
    PreOutcome = pdb.gimp_layer_new_from_visible(newImage,newImage,
                                          " preliminary outcome")
    newImage.add_layer(PreOutcome,0)
    PreOutcome.mode=LAYER_MODE_NORMAL
    PreOutcome.opacity = 100.0
    
    BGonTop = newBG.copy()
    newImage.add_layer(BGonTop,0)
    BGonTop.name = "BG on Top for mixed colouring "
    BGonTop.mode = LAYER_MODE_LINEAR_LIGHT
    BGonTop.opacity = 40.0  # 60.0   # chg 14.1
    
    if Mask>0:
        maskedBG = newBG.copy()
        newImage.add_layer (maskedBG, 0)
        maskedBG.name = "BG on Top for portraits touch-up "

        maskedBG.mode = LAYER_MODE_NORMAL                                 
        maskedBG.opacity = 90.0                                             
        #                                                                     
        if Mask==1:
            blackmask = pdb.gimp_layer_create_mask (maskedBG, ADD_MASK_BLACK)    
            pdb.gimp_layer_add_mask (maskedBG, blackmask)
            optmask=", black mask"
        if Mask==2:
            graymask = pdb.gimp_layer_create_mask (maskedBG, ADD_MASK_COPY)      
            pdb.gimp_layer_add_mask (maskedBG, graymask)
            optmask=", gray-copy mask"
    else:   optmask=""        
         
    Outcome = pdb.gimp_layer_new_from_visible(newImage,newImage, "")
    newImage.add_layer(Outcome,0)
     
    # check lines direction: if vertical restore rotation
    if vertical==True:
        pdb.gimp_image_rotate (newImage, ROTATE_270)
        vertical=False
     
    if doFlatten==True:
        pdb.gimp_item_set_visible (newBG, False) # temp
        Outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_IMAGE)
        pdb.gimp_item_set_visible (newBG, True) # reset
    else:
        pdb.gimp_item_set_visible (Outcome, False)
        
    linesoption=["curved","horizontal","vertical"]
    optline = ", lines "+str(linesoption[linesdir])
    Outcome.name = st+" "+newBG.name+strokeMode+strokeMode+optline+optmask
    
    if outimgWH==0: outimgWH=OrigWH
    uRatio = outimgWH*1.00/WorkWH*1.00
    uWidth = int(rWidth*uRatio)
    uHeight = int(rHeight*uRatio)
    #info_msg ("SCALING")   # not during tests, scaling is slow
    pdb.gimp_image_scale (newImage, uWidth, uHeight)   # to user chosen size
        
    pdb.gimp_display_new(newImage)
    
    # tier down
    
    pdb.gimp_selection_none(newImage)
    pdb.gimp_context_pop()
    
    #pdb.gimp_display_new(newImage)
     
    return(Outcome)

# This is the plugin registration function

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/Drawing/"
copyrightYear="2021"
desc="Draw Effect Style G (v.2) "

register(
    "DrawStyleG",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+" ...",
    "*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of New Image"+"\n (0=same as input)",+
                              3500, (0, 8000, 100)),
      (PF_OPTION, "maskbg", "Add Masked BG on Top?"+
                    "\n  (for portraits adjustments)", 2, ["*none",
                                                            "black (user paints over)",
                                                            "gray copy"]),
      (PF_OPTION, "lines", "Lines direction?", 0, ["*none (curved)",
                                                   "horizontal",
                                                   "vertical"]),
      (PF_TOGGLE, "flatten", "Flatten image?", False),
    ], 
    [
      (PF_DRAWABLE, "drawable", "Output drawable", None),
    ],
    DrawStyleG,
    menu=menu
    )

main()
